// This sketch requires the HC-05 module to start in the AT command mode.
// 1. Disconnect the VCC (power) line from the HC-05 and reconnect it while holding down the button.
// 2. The HC-05 LED should blink on and off at about 2 second intervals. Otherwise, try Step 1 again.
// 3. Modify the values of "hc05_name" & "hc05_password" below, if you want to change them.
// 4. Upload this sketch.
// 5. Open the serial monitor (CTRL+SHIFT+M) and choose "Both NL & CR" & "9600 baud".
// 6. Wait until "Setup complete!!!" is displayed

// Some AT commands, like AT+NAME?, require the button to be held down during execution.
 // Renaming the module (AT+NAME=<new name>) doesn't seem to require the button press though.
 
#include <SoftwareSerial.h>

SoftwareSerial hc05(2, 3);  // Rx, Tx
// Connect Arduino pin 2 (RX) to the HC-05 TX.
// Connect Arduino pin 3 (TX) to the HC-05 RX through a voltage divider.

long hardserial_baudrate = 9600;   // connection speed between PC and Arduino
long softserial_baudrate = 38400;  // connection speed between Arduino and HC-05 in the AT command mode

String hc05_name = "RewardDevice001";
String hc05_password = "1234";
long hc05_baudrate = 38400;         // connection speed between Arduino and HC-05 in the communication mode
long hc05_stopbits = 0;                    // 1 stop bit
long hc05_parity = 0;                        // no parity
bool bReady = false;

bool send_and_ok(String command, bool bVerbose = true);
String send_and_receive(String command, bool bVerbose = true);

void setup() {
  Serial.begin(hardserial_baudrate);  // between PC and Arduino
  hc05.begin(softserial_baudrate);      // between Arduino and HC-05

  for (size_t m=0; m<10 & false==bReady; m++) {
    bReady = send_and_ok("AT", false);
    delay(100);
  }
  Serial.println(bReady ? "Connected successfully!!!" : "Connection failed!!!");

  if (bReady)
  {
    Serial.println("\n* Previous settings:");
    send_and_receive("AT+ROLE?");  // slave (0) or master (1)
    send_and_receive("AT+NAME?");  // This line will return blank, if the button is not being pressed
    send_and_receive("AT+PSWD?");
    send_and_receive("AT+UART?");

    Serial.println("\n* New settings:");
    send_and_ok("AT+ORGL", false);  // restore default
    send_and_ok("AT+ROLE=0");       // slave mode
    send_and_ok(String("AT+NAME=") + hc05_name);
    send_and_ok(String("AT+PSWD=") + hc05_password);
    send_and_ok(String("AT+UART=") + hc05_baudrate + "," + hc05_stopbits + "," + hc05_parity);
    
    Serial.println("\nSetup complete!!!");
  }
}

void loop() {
}

bool send_and_ok(String command, bool bVerbose) {
  command.trim();
  if (bVerbose) Serial.print(command);
  command.concat("\r\n");
  hc05.write(command.c_str());
  
  String result = hc05.readString();
  result.trim();
  if (bVerbose) Serial.println(" " + result);
  
  return result.equals("OK");
}

String send_and_receive(String command, bool bVerbose) {
  command.trim();
  command.concat("\r\n");
  hc05.write(command.c_str());
  
  String result = hc05.readString();
  long to = result.indexOf("OK");
  if (-1==to) to = result.length();
  result = result.substring(0,to);
  result.trim();

  if (bVerbose) Serial.println(result.c_str());
  
  return result;
}
