classdef mlconditions < handle
    properties (SetAccess = protected)
        Conditions
        UIVars
    end
    properties (SetAccess = protected, Hidden)
        MLPath
    end
    
    methods
        function obj = mlconditions(ConditionsFile)
            obj.MLPath = mlpath(mfilename('fullpath'));
            init(obj);
            if exist('ConditionsFile','var'), load_file(obj,ConditionsFile); end
        end
        function init(obj)
            obj.Conditions = [];
            obj.UIVars.TotalNumberOfConditions = [];
            obj.UIVars.StimulusList = '';
            obj.UIVars.BlockList = [];
            obj.UIVars.TotalNumberOfConditionsInThisBlock = [];
            obj.UIVars.TimingFiles = '';
        end
        function val = isloaded(obj), val = ~isempty(obj.Conditions); end
        function val = isconditionsfile(obj), val = isstruct(obj.Conditions); end
        function val = isuserloopfile(obj), val = ischar(obj.Conditions); end
        
        function load_file(obj,ConditionsFile,wb)
            if ~exist('ConditionsFile','var') || 2~=exist(ConditionsFile,'file')
                [n,p] = uigetfile({'*.txt', 'Conditions file (*.txt)'; '*.m', 'Userloop function (*.m)'},'Select a Conditions File');
                if 0==n, error('Can''t find the conditions file.'); end
                ConditionsFile = [p n];
            end
            if isempty(fileparts(ConditionsFile)), ConditionsFile = which(ConditionsFile); end
            obj.MLPath.ConditionsFile = ConditionsFile;
            init(obj);
            
            a = strtrim(regexp(fileread(ConditionsFile),'[^\n]+\n|[^\n]+$','match'));
            b = ~cellfun(@isempty,a);
            cond = cellfun(@obj.unquote,strtrim(regexp(a(b),'\t+','split')'),'UniformOutput',false);
            % in case that the selected file is a userloop file
            if strncmp(cond{1}{1},'function ',9), obj.Conditions = ConditionsFile; end
            if 1==length(cond) && 2==exist(cond{1}{1},'file'), obj.Conditions = which(cond{1}{1}); end
            if ~isempty(obj.Conditions)
                obj.UIVars.StimulusList = {'user-defined'};
                obj.UIVars.TimingFiles = {'user-defined'};
                return
            end
            
            % normal conditions file
            header = {'Condition','Timing File','Frequency','Block','Info'};
            taskobj_no = cellfun(@obj.token2num,regexp(cond{1},'TaskObject#(\d+)','tokens'));
            non_taskobj = cond{1}(isnan(taskobj_no));
            known_header = ismember(lower(non_taskobj),lower(header));
            
            if all(isnan(taskobj_no)), error('No TaskObject header found'); end
            if any(~known_header), error('Invalid header column, ''%s''',non_taskobj{find(~known_header,1)}); end
            
            nheader = length(header);
            col = zeros(1,nheader);
            for m=1:nheader
                idx = find(strcmp(header{m},cond{1}),1);
                if ~isempty(idx), col(m) = idx; end
            end
            
            if any(0==col(1:4)), error('Missging header column, ''%s''',header{find(0==col(1:4),1)}); end
            
            ncond = size(cond,1)-1;
            taskobj = cell(ncond,7);
            for m=1:ncond
                if exist('wb','var'), set(wb,'string',sprintf('Loading... %.0f%%',m/ncond*100)); drawnow; end
                taskobj(m,0~=col) = cond{m+1}(col(0~=col));
                taskobj{m,6} = eval(['struct(' taskobj{m,5} ');']);
                
                idx = taskobj_no(1:length(cond{m+1}));
                obj_no = idx(~isnan(idx));
                taskobj{m,7} = struct('Label',[],'Attribute',[]);
                taskobj{m,7}(max(obj_no)).Label = '';
                a = parse_object(obj,cond{m+1}(~isnan(idx)),false);
                if isempty(a), error('error(s) in parsing %s',ConditionsFile); end
                taskobj{m,7}(obj_no) = a;
                
                taskobj{m,1} = str2double(taskobj{m,1});
                if m~=taskobj{m,1}, error('Condition numbers are not in order from 1 to %d',ncond); end
                [~,taskobj{m,2}] = find_ext(obj,taskobj{m,2},{'.m'});
                taskobj{m,3} = str2double(taskobj{m,3});
                taskobj{m,4} = eval(regexprep(regexprep(taskobj{m,4},' ',','),'^\[?([\d,]+)]?','[$1]'));
                
                if isempty(taskobj{m,2}), error('Can''t find the timing file, ''%s''',taskobj{m,2}); end
                if isnan(taskobj{m,3}) || taskobj{m,3}<0, error('''Frequency'' must be 0 or a natural number (0 or larger)'); end
                if any(isnan(taskobj{m,4})) || any(taskobj{m,4}<1), error('''Block'' must be natural numbers (1 or larger)'); end
            end
            if 0==ncond, error('No condition defined in the file'); end
            
            val = cell2struct(taskobj,{'Condition','TimingFile','Frequency','Block','RawInfo','Info','TaskObject'},2);
            
            all_stim = [val.TaskObject];
            [~,ia] = unique({all_stim.Label});
            unique_stim = all_stim(ia);
            test_file_stimuli(obj,unique_stim);

            obj.UIVars.TotalNumberOfConditions = length(val);
            obj.UIVars.StimulusList = unique_stim;
            obj.UIVars.BlockList = unique([val.Block]);
            obj.UIVars.TotalNumberOfConditionsInThisBlock = hist([val.Block],obj.UIVars.BlockList); %#ok<HIST>
            [obj.UIVars.TimingFiles,~,obj.UIVars.TimingFilesNo] = unique({val.TimingFile}');
            
            obj.Conditions = val;
        end
        
        function val = parse_object(obj,str,file_check)
            if ~exist('file_check','var'), file_check = true; end
            if ~iscell(str), str = {str}; end
            nstr = length(str);
            a = cell(nstr,2);
            for m=1:nstr
                switch lower(str{m}(1:3))
                    case {'fix','dot'}  % fix(Xpos,Ypos)
                        tokens = regexp(str{m},'^([a-zA-Z]{3}) *\( *(-? *(?:\.\d+|\d+(\.\d*)?)) *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *)','tokens');
                        if ~isempty(tokens), tokens{1}(2:3) = str2num(obj,tokens{1}(2:3)); b = 'FIX: default'; end %#ok<*ST2NM>
                    case 'pic'  % pic(filename,Xpos,Ypos[,colorkey]) or pic(filename,Xpos,Ypos,width,height[,colorkey])
                        tokens = regexp(str{m},['^([a-zA-Z]{3}) *\( *''?([^'',]+)''? *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *(?:, *)?(\[ *(?:\.\d+|\d+(\.\d*)?) *,? *(?:\.\d+|\d+(\.\d*)?) *,? *(?:\.\d+|\d+(\.\d*)?) *\])?(?: *)?\)' ...
                            '|^([a-zA-Z]{3}) *\( *''?([^'',]+)''? *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *, *(\.\d+|\d+(?:\.\d*)?) *, *(\.\d+|\d+(?:\.\d*)?) *(?:, *)?(\[ *(?:\.\d+|\d+(\.\d*)?) *,? *(?:\.\d+|\d+(\.\d*)?) *,? *(?:\.\d+|\d+(\.\d*)?) *\])?(?: *)?\)'],'tokens');
                        if ~isempty(tokens)
                            [p,n] = find_ext(obj,tokens{1}{2},{'.png','.jpg','.jpeg','.bmp','.tif','.tiff','.gif'});
                            if isempty(p), error('PIC: Can''t find ''%s''',tokens{1}{2}); else, tokens{1}{2} = p; end
                            if isempty(tokens{1}{end}), tokens{1} = tokens{1}(1:end-1); end
                            tokens{1}(3:end) = str2num(obj,tokens{1}(3:end));
                            if length(tokens{1})<6, b = sprintf('PIC: %s',n); else, tokens{1}(5:6) = num2cell(round([tokens{1}{5:6}])); b = sprintf('PIC: %s [%d x %d]',n,tokens{1}{5:6}); end
                            if 3==length(tokens{1}{end}), b = [b sprintf(', colorkey(%1.2g, %1.2g %1.2g)',tokens{1}{end})]; end %#ok<AGROW>
                        end
                    case 'mov'  % mov(filename,Xpos,Ypos[,looping])
                        tokens = regexp(str{m},'^([a-zA-Z]{3}) *\( *''?([^'',]+)''? *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *(?:, *)?(\d+)?(?: *)?\)','tokens');
                        if ~isempty(tokens)
                            [p,n] = find_ext(obj,tokens{1}{2},{'.3g2','.3gp','.3gp2','.3gpp','.asf','.wmv','.m4a','.m4v','.mov','.mp4','.avi','.mpg','.mpeg','.gif'});
                            if isempty(p), error('MOV: Can''t find ''%s''',tokens{1}{2}); else, tokens{1}{2} = p; end
                            if isempty(tokens{1}{end}), tokens{1} = tokens{1}(1:end-1); end
                            tokens{1}(3:end) = str2num(obj,tokens{1}(3:end));
                            b = sprintf('MOV: %s',n);
                        end
                    case 'crc'  % crc(radius,RGB,fill,Xpos,Ypos)
                        tokens = regexp(str{m},'^([a-zA-Z]{3}) *\( *(\.\d+|\d+(?:\.\d*)?) *, *(\[ *(?:\.\d+|\d+(\.\d*)?) *,? *(?:\.\d+|\d+(\.\d*)?) *,? *(?:\.\d+|\d+(\.\d*)?) *\]) *, *(\d+(\.\d*)?) *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *\)','tokens');
                        if ~isempty(tokens)
                            tokens{1}(2:end) = str2num(obj,tokens{1}(2:end));
                            if any(0==tokens{1}{2}), error('CRC: Size cannot be zero'); end
                            b = sprintf('CRC: %s r=%1.2g rgb=[%1.2g %1.2g %1.2g]',fi(obj,0<tokens{1}{4},'Solid','Outline'),tokens{1}{2:3});
                        end
                    case 'sqr'  % sqr(size,RGB,fill,Xpos,Ypos)
                        tokens = regexp(str{m},['^([a-zA-Z]{3}) *\( *(\.\d+|\d+(?:\.\d*)?) *, *(\[ *(?:\.\d+|\d+(\.\d*)?) *,? *(?:\.\d+|\d+(\.\d*)?) *,? *(?:\.\d+|\d+(\.\d*)?) *\]) *, *(\d+(\.\d*)?) *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *\)' ...
                            '|^([a-zA-Z]{3}) *\( *(\[ *(?:\.\d+|\d+(\.\d*)?) *,? *(?:\.\d+|\d+(\.\d*)?) *\]) *, *(\[ *(?:\.\d+|\d+(\.\d*)?) *,? *(?:\.\d+|\d+(\.\d*)?) *,? *(?:\.\d+|\d+(\.\d*)?) *\]) *, *(\d+(\.\d*)?) *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *\)'],'tokens');
                        if ~isempty(tokens)
                            tokens{1}(2:end) = str2num(obj,tokens{1}(2:end));
                            if any(0==tokens{1}{2}), error('SQR: Size cannot be zero'); end
                            if 1==length(tokens{1}{2}), tokens{1}{2} = [tokens{1}{2} tokens{1}{2}]; end
                            b = sprintf('SQR: %s [%1.2g x %1.2g] rgb=[%1.2g %1.2g %1.2g]',fi(obj,0<tokens{1}{4},'Solid','Outline'),tokens{1}{2:3});
                        end
                    case 'snd'  % snd(filename) or snd(sin,duration,frequency)
                        tokens = regexp(str{m},'^([a-zA-Z]{3}) *\( *''?([^'',]+)''? *\)|^([a-zA-Z]{3}) *\( *''?(?i:sin)''? *, *(\.\d+|\d+(?:\.\d*)?) *, *(\.\d+|\d+(?:\.\d*)?) *\)','tokens');
                        if ~isempty(tokens)
                            if 2==length(tokens{1})
                                [p,n] = find_ext(obj,tokens{1}{2},{'.wav','.mat','.m4a','.aac','.mp3','.ogg','.mp4','.avi'});
                                if isempty(p), error('SND: Can''t find ''%s''',tokens{1}{2}); else, tokens{1}{2} = p; end
                                b = sprintf('SND: %s',n);
                            else
                                tokens{1}(2:end) = str2num(obj,tokens{1}(2:end));
                                b = sprintf('SND: sine %g kHz, %g s',tokens{1}{3}/1000,tokens{1}{2});
                            end
                        end
                    case 'stm'  % stm(port,datasource[,retriggerable])
                        tokens = regexp(str{m},'^([a-zA-Z]{3}) *\( *(\d+) *, *''?([^'',]+)''? *(?:, *)?(\d+)?(?: *)?\)','tokens');
                        if ~isempty(tokens)
                            [p,n] = find_ext(obj,tokens{1}{3},{'.wav','.mat','.m4a','.aac','.mp3','.ogg','.mp4','.avi'});
                            if isempty(p), error('STM: Can''t find ''%s''',tokens{1}{3}); else, tokens{1}{3} = p; end
                            if isempty(tokens{1}{end}), tokens{1} = tokens{1}(1:end-1); end
                            if 3==length(tokens{1}), tokens{1}(2) = str2num(obj,tokens{1}(2)); tokens{1}{4} = 0; else, tokens{1}([2 4]) = str2num(obj,tokens{1}([2 4])); end
                            b = sprintf('STM: #%d%s, %s',tokens{1}{2},fi(obj,0<tokens{1}{4},' Retriggerable',''),n);
                        end
                    case 'ttl'  % ttl(port)
                        tokens = regexp(str{m},'^([a-zA-Z]{3}) *\( *(\d+) *\)','tokens');
                        if ~isempty(tokens), tokens{1}(2) = str2num(obj,tokens{1}(2)); b = sprintf('TTL: #%d',tokens{1}{2}); end
                    case 'gen'  % gen(function_name) or gen(function_name,Xpos,Ypos)
                        tokens = regexp(str{m},'^([a-zA-Z]{3}) *\( *''?([^'',]+)''? *)|^([a-zA-Z]{3}) *\( *''?([^'',]+)''? *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *, *(-? *(?:\.\d+|\d+(\.\d*)?)) *\)','tokens');
                        if ~isempty(tokens)
                            [p,n] = find_ext(obj,tokens{1}{2},{'.m'});
                            if isempty(p), error('GEN: Can''t find ''%s''',tokens{1}{2}); else, tokens{1}{2} = p; end
                            if 2<length(tokens{1}), tokens{1}(3:4) = str2num(obj,tokens{1}(3:4)); end
                            b = sprintf('GEN: %s',n);
                        end
                    otherwise, error('Unknown object type, %s',str{m});
                end
                if isempty(tokens), error('Can''t parse out ''%s''',str{m}); end
                a{m,1} = b; a(m,2) = tokens(1);
            end
            val = cell2struct(a,{'Label','Attribute'},2)';
            if file_check, test_file_stimuli(obj,val); end
        end
        function test_file_stimuli(~,a)
            for m=1:length(a)
                tokens = a(m).Attribute;
                switch lower(tokens{1})
                    case 'pic', p = tokens{2}; [~,n] = fileparts(p); try mglimread(p); catch err, error('PIC: %s (%s)',err.message, n); end
                    case 'mov', p = tokens{2}; [~,n] = fileparts(p); try mgldestroygraphic(mgladdmovie(p,0)); catch err, error('MOV: %s (%s)',err.message,n); end
                    case 'snd'
                        if 2==length(tokens)
                            p = tokens{2}; [~,n] = fileparts(p); try y = load_waveform(tokens); if isscalar(y), mgldestroysound(y); end, catch err, error('SND: %s (%s)',err.message,n); end
                        end
                    case 'stm', p = tokens{3}; [~,n] = fileparts(p); try load_waveform({'stm',p}); catch err, error('STM: %s (%s)',err.message,n); end
                    case 'gen', p = tokens{2}; [~,n] = fileparts(p); try get_function_handle(p); catch err, error('GEN: %s (%s)',err.message,n); end
                end
            end
        end
        
        function [p,n] = find_ext(obj,name,ext_list)
            [p,n,e] = fileparts(name); name = [n e];
            if ~isempty(p)
                if filesep~=p(end), p = [p filesep]; end
                name = [p name];
            end
            if ischar(ext_list), ext_list = {ext_list}; end
            if any(strcmpi(e,ext_list)), ext_list = [{''} ext_list]; end
            for ext = ext_list(:)'
                [p,n] = mlsetpath([name ext{1}],{obj.MLPath.ExperimentDirectory,obj.MLPath.BaseDirectory,[obj.MLPath.BaseDirectory 'mgl']});
                if ~isempty(p), [~,n,e] = fileparts(n); n = [n lower(e)]; return, end %#ok<AGROW>
            end
            p = ''; n = '';
        end
        function op = fi(~,tf,op1,op2)
            if tf, op = op1; else, op = op2; end
        end
        function num = str2num(~,str)
            num = cellfun(@eval,str,'UniformOutput',false);
        end
        function val = token2num(~,token)
            if isempty(token), val = NaN; else, val = str2double(token{1}); end
        end
        function val = unquote(~,str)
            nstr = length(str);
            val = cell(1,nstr);
            for m=1:nstr
                l = length(str{m});
                if l<2
                    val(m) = str(m);
                elseif 2==l
                    if strcmp(str{m},'""') || strcmp(str{m},''''''), val{m} = ''; else, val(m) = str(m); end
                else
                    s1 = str{m}([1 l]);
                    s2 = str{m}(2:l-1);
                    if strcmp(s1,'""') && isempty(regexp(s2,'"','once')) || strcmp(s1,'''''') && isempty(regexp(s2,'''','once')), val{m} = s2; else, val(m) = str(m); end
                end
            end
        end
    end
end
